########################
#
# GAB isotherm model- This script is made freely available and
# this notice should be included with the distribution of this script.
# This script may be freely distributed and modified.  Jeff Kenvin,
# January 2014
#
########################
import mic
import numpy as np
########
#
# create color table - this is a convenient table - it may not suit your sense of aesthetics.
#
black = "#000000"
blue = "#0000FF"
green = "#008000"
red = "#FF0000"
#
navy = "#000080"
teal = "#008080"
lime = "#00FF00"
aqua = "#00FFFF"
purple = "#800080"
gray = "#808080"
#
########
#
## data from sample file
p_rel, q_ads, n_ads, warm_fs, cold_fs, mass, desc = mic.isotherm('rel')
csa, hsd, dcf, mol_weight, analysis_gas = mic.adsorptive_data(0)
#
## the n_ads is the number of adsorption points
p_rel = p_rel[0 : n_ads]            # only adsorption points for relative pressure
q_ads = q_ads[0 : n_ads]            # only adsorption points for volume adsorbed
#
x = p_rel
y = q_ads
# 
xMin = 0.0
xMax = 1.0

# select a subset of data
iFit=np.nonzero(np.logical_and(x>xMin,x<xMax))
xf = (x.take(iFit))
yf =(y.take(iFit))
# print x.take(iFit)

# GAB is (P/Po)/nAds = alpha + beta*(P/Po) + gamma*(P/Po)^2
# create matrix for Ax=b
yG = xf/yf
ix =np.array( np.ones(xf.shape))
A = np.vstack([xf**2,xf,ix]).T
#
## from the GAB equation g is gamma, b is beta, and a is alpha
g,b,a = np.linalg.lstsq(A,yG.T)[0]
R = np.corrcoef(xf,yG)

# k1 is usually the correct root
k1 = (-b + (b**2 - 4*a*g)**0.5)/2/a; 
k2 = (-b - (b**2 - 4*a*g)**0.5)/2/a;
k = np.maximum(k1,k2)
c = 2 + b/k/a;
# calculate monolayer volume
vm = 1/k/c/a;

#  line for GAB model
yp = x/y/(1-k*x)
xT = np.array([xMin,xMax])
yT = 1/vm/c/k + (c-1)/c/vm*xT
yTo =  1/vm/c/k + (c-1)/c/vm*x
vC = vm*k*c*x/(1-k*x)/(1-k*x*(1-c))
#
#########
#
# send data back to the MicroActive UI and Report system
#
#  Create a summary report - this will be displayed in MicroActive UI and Report system
mic.summary('GAB Summary')
bet = vm/22414*6.02214129e23*csa/1.0e18
if vm < 0:
    mic.summary.add('GAB parameters',['GAB Surface Area = ',
                                      'ERROR Negative Vm value !! = ', 'C = ', 'k = ' ,
                                      'alpha = ', 'beta = ', 'gamma =' ],
        [ "%8.3f" % bet, "%8.3f" % vm , "%8.3f" % c, "%8.3f" % k ,
          "%8.3f" % a, "%8.3f" % b, "%8.3f" % g ])
else:
    mic.summary.add('GAB parameters',['GAB Surface Area, m2/g = ','Vm, cm3/g = ', 'C = ', 'k = ',
                                      'alpha = ', 'beta = ', 'gamma ='],
                    [ "%8.3f" % bet, "%8.3f" % vm , "%8.3f" % c, "%8.3f" % k,
                    "%8.3f" % a, "%8.3f" % b, "%8.3f" % g ])
#
# create plots
mic.graph('GAB Model','a','a/(V(1-ka))')
mic.graph.add('Data',x,yp, graphtype='points',color=black)
mic.graph.add('Fit',xT,yT, linestyle='-',graphtype='curve',color = blue)
#
# a second use of mic.graph creates another plot
mic.graph('GAB','activity','quantity adsorbed')
mic.graph.add('Data',x,y, graphtype='points',color=black)
mic.graph.add('Fit',x,vC, linestyle='-',graphtype='curve',color = red)
#
# create a tabular report
mic.table(title = 'GAB Report')
mic.table.addtable( 'Tabular Report')
mic.table.addcolumn('Activity' , [("%6.3f" % vOut) for vOut in x] )
mic.table.addcolumn('Qty adsorbed, cm3/g' , [ ("%6.3f" % vOut) for vOut in y] )
mic.table.addcolumn('GAB (exp), g/cm3' , [("%6.3f" % vOut) for vOut in yp] )
mic.table.addcolumn('GAB (calc),g/cm3' , [("%6.3f" % vOut) for vOut in yTo] )
