########################
#
# Langmuir dissociative ads isotherm model- This script is made freely
# available and this notice should be included with the distribution
# of this script.  This script may be freely distributed and modified.
# Jeff Kenvin, May 2013
#
########################
import numpy as np
import mic
#

## data from sample file
p_ads, q_ads, n_ads, warm_fs, cold_fs, mass, desc = mic.isotherm('abs')

# function to convert to mbar
def fmmHg2mbar( p ):
    pmBar = p * 1013.25 / 760.0
    return pmBar
# function to convert to mmoles
def fV2mmol( v ):
    nmmol = v/22.414
    return nmmol
#
x = fmmHg2mbar(p_ads)
xMin = np.min(x)
xMax = np.max(x)
xStep = (xMax-xMin)/100.0
xF = np.arange(xMin,xMax,xStep)
# convert to milli moles
y = fV2mmol(q_ads)
#
# calculate Langmuir transform
yL = (x**0.5/y)
#
# create matrix for Ax=b
ix =np.array( np.ones(x.shape))
A = np.vstack([x**0.5,ix]).T
#
m,b = np.linalg.lstsq(A,yL.T)[0]
R = np.corrcoef(x,yL)
nm = 1/m
c =  (m/b)**2.0
K = nm*c
#
xP = np.arange(xMin,xMax,(xMax-xMin)/25.0)
yP = m*xP**0.5 + b
yOut = m*x**0.5 + b
yOut = x**0.5/yOut
##################################
mic.summary('Summary')
mic.summary.add( 'Langmuir dual site parameters', ['b, 1/mbar = ', 
                                     'Qm, mmol/g = ',
                                     ],
                 [ "%10.5f" % c,
                  "%10.5f" % nm, 
                 ] )
#
# create color table - this is a convenient table - it may not suit
# your sense of aesthetics.
#
black = "#000000"
blue = "#0000FF"
green = "#008000"
red = "#FF0000"
#
navy = "#000080"
teal = "#008080"
lime = "#00FF00"
aqua = "#00FFFF"
purple = "#800080"
gray = "#808080"
#
########
#
mic.graph('Fit', '\u221A'+'P'+'\u0305','\u221A'+'P'+'\u0305'+'/n', xlinear = 1, ylinear = 1)
mic.graph.add('Isotherm', x**0.5 , yL, color = red,graphtype = 'points', marker = 'o')
mic.graph.add('Langmuir DS', xP**0.5, yP ,color = blue,  linestyle = '-', graphtype = 'curve')
#
mic.graph('Model','Pressure, mbar','Qads, mmoles/g')
mic.graph.add('Isotherm', x , y,  color=red, graphtype = 'points', marker = 'o')
mic.graph.add('Langmuir DS', xP, xP**0.5/yP ,color = blue,  linestyle = '-', graphtype = 'curve')

#
# create tabular output
#
#########
#
# send data back to the MicroActive UI and Report system
#
#
#
def fTextOut( x, oFormat):
    tOut = [(oFormat % xOut) for xOut in x]
    return tOut
#
mic.table(title='Tabular Report')
mic.table.addtable('Tabular Report')
mic.table.addcolumn('P, mbar', fTextOut( x, '%6.4f'))
mic.table.addcolumn('Qads, mmol/g', fTextOut(y, '%8.3f'))
mic.table.addcolumn('Q(DS), mmol/g', fTextOut(yOut, '%8.3f'))
mic.table.addcolumn('Residual, mmol/g', fTextOut((y-yOut), '%8.3f'))
mic.table.addcolumn('% Different', fTextOut((y-yOut)/y*100.0, '%8.3f'))

